@@ -152,18 +152,6 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    applyopts_offset(xfd, opts);
    applyopts_cloexec(xfd->fd, opts);
 
-#if defined(WITH_VSOCK) && defined(IOCTL_VM_SOCKETS_GET_LOCAL_CID)
-   {
-      unsigned int cid;
-      if (Ioctl(xfd->fd, IOCTL_VM_SOCKETS_GET_LOCAL_CID, &cid) < 0) {
-	 Warn2("ioctl(%d, IOCTL_VM_SOCKETS_GET_LOCAL_CID, ...): %s",
-	       xfd->fd, strerror(errno));
-      } else {
-	 Notice1("VSOCK CID=%u", cid);
-      }
-   }
-#endif
-
    applyopts(xfd->fd, opts, PH_PREBIND);
    applyopts(xfd->fd, opts, PH_BIND);
    if (Bind(xfd->fd, (struct sockaddr *)us, uslen) < 0) {
@@ -176,7 +164,11 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 
 #if WITH_UNIX
    if (us->sa_family == AF_UNIX) {
-      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_FD);
+      if (((union sockaddr_union *)us)->un.sun_path[0] != '\0') {
+	 applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_FD);
+      } else {
+	 applyopts(xfd->fd, opts, PH_FD);
+      }
    }
 #endif
    /* under some circumstances (e.g., TCP listen on port 0) bind() fills empty
@@ -190,9 +182,14 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    applyopts(xfd->fd, opts, PH_PASTBIND);
 #if WITH_UNIX
    if (us->sa_family == AF_UNIX) {
-      /*applyopts_early(((struct sockaddr_un *)us)->sun_path, opts);*/
-      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_EARLY);
-      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_PREOPEN);
+      if (((union sockaddr_union *)us)->un.sun_path[0] != '\0') {
+	 /*applyopts_early(((struct sockaddr_un *)us)->sun_path, opts);*/
+	 applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_EARLY);
+	 applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_PREOPEN);
+      } else {
+	 applyopts(xfd->fd, opts, PH_EARLY);
+	 applyopts(xfd->fd, opts, PH_PREOPEN);
+      }
    }
 #endif /* WITH_UNIX */
 
