@@ -1,5 +1,5 @@
 /* source: xio-named.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for filesystem entry functions */
@@ -24,7 +24,7 @@ const struct optdesc opt_unlink_close  = { "unlink-close", NULL, OPT_UNLINK_CLOS
 const struct optdesc opt_umask       = { "umask",       NULL, OPT_UMASK,       GROUP_NAMED, PH_EARLY,  TYPE_MODET, OFUNC_SPEC };
 #endif /* WITH_NAMED */
 
-/* applies to fd all options belonging to phase */
+/* applies to filesystem entry all options belonging to phase */
 int applyopts_named(const char *filename, struct opt *opts, unsigned int phase) {
    struct opt *opt;
 
@@ -96,7 +96,6 @@ int _xioopen_named_early(int argc, const char *argv[], xiofile_t *xfd,
 			 int groups,
 		      bool *exists, struct opt *opts) {
    const char *path = argv[1];
-   unsigned int iogroups = 0;
 #if HAVE_STAT64
    struct stat64 statbuf;
 #else
@@ -120,10 +119,8 @@ int _xioopen_named_early(int argc, const char *argv[], xiofile_t *xfd,
 	 Error2("stat(\"%s\"): %s", path, strerror(errno));
 	    return STAT_RETRYLATER;
       }
-      iogroups = GROUP_REG;
       *exists = false;
    } else {
-      iogroups = _groupbits(statbuf.st_mode);
       *exists = true;
    }
 
@@ -135,14 +132,13 @@ int _xioopen_named_early(int argc, const char *argv[], xiofile_t *xfd,
       Info1("\"%s\" already exists; removing it", path);
       if (Unlink(path) < 0) {
 	 Error2("unlink(\"%s\"): %s", path, strerror(errno));
-	 *exists = true;
       } else {
 	 *exists = false;
       }
    }
 
-   applyopts(-1, opts, PH_EARLY);
    applyopts_named(path, opts, PH_EARLY);
+   applyopts(-1, opts, PH_EARLY);
    if (*exists) {
       applyopts_named(path, opts, PH_PREOPEN);
    } else {
