@@ -1,5 +1,5 @@
-/* $Id: xiosysincludes.h,v 1.4 2001/12/27 16:52:34 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001 */
+/* source: xiosysincludes.h */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xiosysincludes_h_included
@@ -9,5 +9,7 @@
 #include "xioconfig.h"	/* what features are enabled */
 
 #include "sysincludes.h"
+//#undef IP_ADD_SOURCE_MEMBERSHIP
+#undef HAVE_STRUCT_IP_MREQ_SOURCE
 
 #endif /* !defined(__xiosysincludes_h_included) */
